!-------------------------------------------------------------------------------

!                      Code_Saturne version 4.0.3-patch
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2015 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_postprocess_var.f90
!> \brief Output additional variables on a postprocessing mesh.
!>
!> Several "automatic" postprocessing meshes may be defined:
!> - The volume mesh (ipart=-1)
!> - The boundary mesh (ipart=-2)
!> - SYRTHES coupling surface (ipart < -2)
!> - Cooling tower exchange zone meshes (ipart < -2) if 'ichrze' = 1
!>
!> Additional meshes (cells or faces) may also be defined through the GUI or
!> using the cs_user_postprocess_meshes() function from the
!> cs_user_postprocess.c file.
!>
!> This subroutine is called once for each post-processing mesh
!> (with a different value of 'ipart') for each time step at which output
!> on this mesh is active.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     ipart         number of the post-processing mesh (< 0 or > 0)
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     nvlsta        number of Lagrangian statistical variables
!> \param[in]     ncelps        number of cells in post-processing mesh
!> \param[in]     nfacps        number of interior faces in post-process. mesh
!> \param[in]     nfbrps        number of boundary faces in post-process. mesh
!> \param[in]     itypps        global presence flag (0 or 1) for cells (1),
!>                              interior faces (2), or boundary faces (3) in
!>                              post-processing mesh
!> \param[in]     lstcel        list of cells in post-processing mesh
!> \param[in]     lstfac        list of interior faces in post-processing mesh
!> \param[in]     lstfbr        list of boundary faces in post-processing mesh
!_______________________________________________________________________________

subroutine usvpst &
 ( ipart  ,                                                       &
   nvar   , nscal  , nvlsta ,                                     &
   ncelps , nfacps , nfbrps ,                                     &
   itypps ,                                                       &
   lstcel , lstfac , lstfbr )

!===============================================================================

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use pointe
use entsor
use optcal
use numvar
use parall
use period
use mesh
use field
use field_operator
use post
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer          ipart
integer          nvar,   nscal , nvlsta
integer          ncelps, nfacps, nfbrps

integer          itypps(3)
integer          lstcel(ncelps), lstfac(nfacps), lstfbr(nfbrps)


! Local variables
character (len=20) :: namevr

integer          ntindp, f_id
integer          iel, ifac, iloc, ivar
integer          idimt, ii , jj, idir, ivel
logical          ientla, ivarpr
integer          imom1, imom2
integer          inc, iprev
double precision pnd
double precision rvoid(1)

double precision, dimension(:), allocatable :: scel, sfac, sfbr
double precision, dimension(:,:), allocatable :: vcel, vfac, vfbr
double precision, allocatable, dimension(:,:,:) :: gradv
double precision, dimension(:), pointer :: bfpro_rom
double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
double precision, dimension(:), pointer :: cvar_var

double precision, dimension(:), pointer :: cmom_1, cmom_2, cdt

double precision, dimension(:,:), pointer :: cvar_vel

integer          intpst
data             intpst /0/
save             intpst

!===============================================================================


!===============================================================================
! Increment call counter once per time step (possibly used in some tests)
!===============================================================================

if (ipart .eq. -1) then
  intpst = intpst + 1
endif

!===============================================================================
! 1. Handle variables to output
!    MUST BE FILLED IN by the user at indicated places
!===============================================================================
!===============================================================================
! Examples for initialization
!===============================================================================

! Pointer to local time step
call field_get_id('dt', f_id)
call field_get_val_s(f_id, cdt)

!===============================================================================
! Examples of volume variables on the main volume mesh (ipart = -1)
!===============================================================================

if (ipart .eq. -1) then
  ! Allocate a temporary array
  allocate(gradv(3,3,ncelet))
  ! Compute of the vel gradient 
  inc = 1
  iprev = 0
  call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc, gradv)
  ! allocation temporary array for definition of the output
  allocate(scel(ncelps))
  ! Loop on local cells   
  do ivel = 1, 3
    do idir = 1,3
      ii = (ivel-1)*3+idir
      do iloc = 1, ncelps
        iel = lstcel(iloc)
        scel(iloc) = gradv(idir,ivel,iel)
      enddo
      ! Print of the results
      do jj = 1, 20
        namevr (jj:jj) = ' '
      enddo
      select case(ii)
        case (1)
          write(namevr,'(A10)') 'GradDv1Dx1'
        case (2)
          write(namevr,'(A10)') 'GradDv1Dx2'
        case (3)
          write(namevr,'(A10)') 'GradDv1Dx3'
        case (4)
          write(namevr,'(A10)') 'GradDv2Dx1'
        case (5)
          write(namevr,'(A10)') 'GradDv2Dx2'
        case (6)
          write(namevr,'(A10)') 'GradDv2Dx3'
        case (7)
          write(namevr,'(A10)') 'GradDv3Dx1'
        case (8)
          write(namevr,'(A10)') 'GradDv3Dx2'
        case (9)
          write(namevr,'(A10)') 'GradDv3Dx3'
      endselect
      idimt = 1        ! 1: scalar, 3: vector, 6/9: symm/non-symm tensor
      ientla = .true.  ! dimension 1 here, so no effect
      ivarpr = .false. ! defined on the work array, not on the parent
      ! Output values; as we have no face values, we can pass a
      ! trivial array rvoid for those.
      call post_write_var(ipart, namevr, idimt, ientla, ivarpr,  &
                          ntcabs, ttcabs, scel, rvoid, rvoid)
    enddo
  enddo

  deallocate(scel)
  deallocate(gradv)

endif ! end of test on post-processing mesh number

return

end subroutine usvpst
